@@ -43,6 +43,71 @@
 
 /*****************************************************************************/
 
+static void
+_test_nmovpn_remote_parse (const char *str,
+                           const char *exp_host,
+                           const char *exp_port,
+                           const char *exp_proto)
+{
+	gs_free char *str_free = NULL;
+	gssize r;
+	const char *host, *port, *proto;
+	gs_free_error GError *error = NULL;
+
+	g_assert (exp_host || (!exp_port && !exp_proto));
+
+	r = nmovpn_remote_parse (str, &str_free, &host, &port, &proto, &error);
+	if (!exp_host) {
+		g_assert (r >= 0);
+		g_assert (error);
+		return;
+	}
+	nmtst_assert_success (r == -1, error);
+
+	g_assert_cmpstr (exp_host, ==, host);
+	g_assert_cmpstr (exp_port, ==, port);
+	g_assert_cmpstr (exp_proto, ==, proto);
+}
+
+static void
+test_nmovpn_remote_parse (void)
+{
+	_test_nmovpn_remote_parse ("a",                          "a",                      NULL,    NULL);
+	_test_nmovpn_remote_parse ("a:",                         "a",                      NULL,    NULL);
+	_test_nmovpn_remote_parse ("t::",                        "t",                      NULL,    NULL);
+	_test_nmovpn_remote_parse ("a::",                        "a::",                    NULL,    NULL);
+	_test_nmovpn_remote_parse ("[a::]:",                     "a::",                    NULL,    NULL);
+	_test_nmovpn_remote_parse ("t:::",                       "t:",                     NULL,    NULL);
+	_test_nmovpn_remote_parse ("a:::",                       "a::",                    NULL,    NULL);
+	_test_nmovpn_remote_parse ("a:t::",                      "a:t",                    NULL,    NULL);
+	_test_nmovpn_remote_parse ("a:b::",                      "a:b::",                  NULL,    NULL);
+	_test_nmovpn_remote_parse ("a::udp",                     "a",                      NULL,    "udp");
+	_test_nmovpn_remote_parse ("a:1:",                       "a",                      "1",     NULL);
+	_test_nmovpn_remote_parse ("t::1:",                      "t:",                     "1",     NULL);
+	_test_nmovpn_remote_parse ("t::1:",                      "t:",                     "1",     NULL);
+	_test_nmovpn_remote_parse ("[a:]:1:",                    "[a:]",                   "1",     NULL);
+	_test_nmovpn_remote_parse ("a::1:",                      "a::1",                   NULL,    NULL);
+	_test_nmovpn_remote_parse ("a::1:1194",                  "a::1:1194",              NULL,    NULL);
+	_test_nmovpn_remote_parse ("[a::1]:1194",                "a::1",                   "1194",  NULL);
+	_test_nmovpn_remote_parse ("a::1194",                    "a::1194",                NULL,    NULL);
+	_test_nmovpn_remote_parse ("a::1194:",                   "a::1194",                NULL,    NULL);
+	_test_nmovpn_remote_parse ("[a:]:1194:",                 "[a:]",                   "1194",  NULL);
+	_test_nmovpn_remote_parse ("a:1:tcp",                    "a",                      "1",     "tcp");
+	_test_nmovpn_remote_parse ("aa:bb::1:1194:udp",          NULL,                     NULL,    NULL);
+	_test_nmovpn_remote_parse ("[aa:bb::1]:1194:udp",        "aa:bb::1",               "1194",  "udp");
+	_test_nmovpn_remote_parse ("[aa:bb::1]::udp",            "aa:bb::1",               NULL,    "udp");
+	_test_nmovpn_remote_parse ("aa:bb::1::udp",              "aa:bb::1",               NULL,    "udp");
+	_test_nmovpn_remote_parse ("aa:bb::1::",                 "aa:bb::1",               NULL,    NULL);
+	_test_nmovpn_remote_parse ("abc.com:1234:udp",           "abc.com",                "1234",  "udp");
+	_test_nmovpn_remote_parse ("ovpnserver.company.com:443", "ovpnserver.company.com", "443",   NULL);
+	_test_nmovpn_remote_parse ("vpn.example.com::tcp",       "vpn.example.com",        NULL,    "tcp");
+	_test_nmovpn_remote_parse ("dead:beef::1:1194",          "dead:beef::1:1194",      NULL,    NULL);
+	_test_nmovpn_remote_parse ("dead:beef::1:1194",          "dead:beef::1:1194",      NULL,    NULL);
+	_test_nmovpn_remote_parse ("2001:dead:beef::1194::",     "2001:dead:beef::1194",   NULL,    NULL);
+}
+
+/*****************************************************************************/
+
 static NMVpnEditorPlugin *
 _create_plugin (void)
 {
@@ -176,9 +241,8 @@ remove_secrets (NMConnection *connection)
 	g_slist_free (keys);
 }
 
-#define PASSWORD_EXPORTED_NAME "password.ovpntest"
 static void
-test_password_export (void)
+test_export_compare (gconstpointer test_data)
 {
 	_CREATE_PLUGIN (plugin);
 	NMConnection *connection;
@@ -186,17 +250,20 @@ test_password_export (void)
 	char *path;
 	gboolean success;
 	GError *error = NULL;
+	const char *file, *exported_name;
 
-	connection = get_basic_connection (plugin, SRCDIR, "password.conf");
+	nmtst_test_data_unpack (test_data, &file, &exported_name);
+
+	connection = get_basic_connection (plugin, SRCDIR, file);
 	g_assert (connection);
 
-	path = g_build_path ("/", TMPDIR, PASSWORD_EXPORTED_NAME, NULL);
+	path = g_build_path ("/", TMPDIR, exported_name, NULL);
 	success = nm_vpn_editor_plugin_export (plugin, path, connection, &error);
 	g_assert_no_error (error);
 	g_assert (success);
 
 	/* Now re-import it and compare the connections to ensure they are the same */
-	reimported = get_basic_connection (plugin, TMPDIR, PASSWORD_EXPORTED_NAME);
+	reimported = get_basic_connection (plugin, TMPDIR, exported_name);
 	(void) unlink (path);
 	g_assert (reimported);
 
@@ -277,6 +344,71 @@ test_tls_import (void)
 	g_object_unref (connection);
 }
 
+static void
+test_tls_import_2 (void)
+{
+	_CREATE_PLUGIN (plugin);
+	NMConnection *connection;
+	NMSettingConnection *s_con;
+	NMSettingVpn *s_vpn;
+	char *expected_path;
+
+	connection = get_basic_connection (plugin, SRCDIR, "tls2.ovpn");
+	g_assert (connection);
+
+	/* Connection setting */
+	s_con = nm_connection_get_setting_connection (connection);
+	g_assert (s_con);
+	g_assert_cmpstr (nm_setting_connection_get_id (s_con), ==, "tls2");
+	g_assert (!nm_setting_connection_get_uuid (s_con));
+
+	/* VPN setting */
+	s_vpn = nm_connection_get_setting_vpn (connection);
+	g_assert (s_vpn);
+
+	/* Data items */
+	_check_item (s_vpn, NM_OPENVPN_KEY_CONNECTION_TYPE, NM_OPENVPN_CONTYPE_TLS);
+	_check_item (s_vpn, NM_OPENVPN_KEY_DEV, "tun");
+	_check_item (s_vpn, NM_OPENVPN_KEY_PROTO_TCP, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_COMP_LZO, "adaptive");
+	_check_item (s_vpn, NM_OPENVPN_KEY_FLOAT, "yes");
+	_check_item (s_vpn, NM_OPENVPN_KEY_RENEG_SECONDS, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE, "173.8.149.245:1194");
+	_check_item (s_vpn, NM_OPENVPN_KEY_PORT, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY_DIRECTION, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_CIPHER, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_LOCAL_IP, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE_IP, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_AUTH, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_TLS_REMOTE, "/CN=myvpn.company.com");
+	_check_item (s_vpn, NM_OPENVPN_KEY_VERIFY_X509_NAME,
+	             "subject:C=US, L=Cambridge, CN=GNOME, emailAddress=networkmanager-list@gnome.org");
+	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE_CERT_TLS, "server");
+
+	expected_path = g_strdup_printf ("%s/keys/mg8.ca", SRCDIR);
+	_check_item (s_vpn, NM_OPENVPN_KEY_CA, expected_path);
+	g_free (expected_path);
+
+	expected_path = g_strdup_printf ("%s/keys/clee.crt", SRCDIR);
+	_check_item (s_vpn, NM_OPENVPN_KEY_CERT, expected_path);
+	g_free (expected_path);
+
+	expected_path = g_strdup_printf ("%s/keys/clee.key", SRCDIR);
+	_check_item (s_vpn, NM_OPENVPN_KEY_KEY, expected_path);
+	g_free (expected_path);
+
+	expected_path = g_strdup_printf ("%s/keys/46.key", SRCDIR);
+	_check_item (s_vpn, NM_OPENVPN_KEY_TLS_CRYPT, expected_path);
+	g_free (expected_path);
+
+	/* Secrets */
+	_check_secret (s_vpn, NM_OPENVPN_KEY_PASSWORD, NULL);
+	_check_secret (s_vpn, NM_OPENVPN_KEY_CERTPASS, NULL);
+
+	g_object_unref (connection);
+}
+
 static void
 test_file_contents (const char *id,
                     const char *dir,
@@ -363,42 +495,6 @@ test_tls_inline_import (void)
 	g_object_unref (connection);
 }
 
-
-#define TLS_EXPORTED_NAME "tls.ovpntest"
-static void
-test_tls_export (void)
-{
-	_CREATE_PLUGIN (plugin);
-	NMConnection *connection;
-	NMConnection *reimported;
-	char *path;
-	gboolean success;
-	GError *error = NULL;
-
-	connection = get_basic_connection (plugin, SRCDIR, "tls.ovpn");
-	g_assert (connection);
-
-	path = g_build_path ("/", TMPDIR, TLS_EXPORTED_NAME, NULL);
-	success = nm_vpn_editor_plugin_export (plugin, path, connection, &error);
-	g_assert_no_error (error);
-	g_assert (success);
-
-	/* Now re-import it and compare the connections to ensure they are the same */
-	reimported = get_basic_connection (plugin, TMPDIR, TLS_EXPORTED_NAME);
-	(void) unlink (path);
-	g_assert (reimported);
-
-	/* Clear secrets first, since they don't get exported, and thus would
-	 * make the connection comparison below fail.
-	 */
-	remove_secrets (connection);
-	g_assert (nm_connection_compare (connection, reimported, NM_SETTING_COMPARE_FLAG_EXACT));
-
-	g_object_unref (reimported);
-	g_object_unref (connection);
-	g_free (path);
-}
-
 static void
 test_pkcs12_import (void)
 {
@@ -457,41 +553,6 @@ test_pkcs12_import (void)
 	g_object_unref (connection);
 }
 
-#define PKCS12_EXPORTED_NAME "pkcs12.ovpntest"
-static void
-test_pkcs12_export (void)
-{
-	_CREATE_PLUGIN (plugin);
-	NMConnection *connection;
-	NMConnection *reimported;
-	char *path;
-	gboolean success;
-	GError *error = NULL;
-
-	connection = get_basic_connection (plugin, SRCDIR, "pkcs12.ovpn");
-	g_assert (connection);
-
-	path = g_build_path ("/", TMPDIR, PKCS12_EXPORTED_NAME, NULL);
-	success = nm_vpn_editor_plugin_export (plugin, path, connection, &error);
-	g_assert_no_error (error);
-	g_assert (success);
-
-	/* Now re-import it and compare the connections to ensure they are the same */
-	reimported = get_basic_connection (plugin, TMPDIR, PKCS12_EXPORTED_NAME);
-	(void) unlink (path);
-	g_assert (reimported);
-
-	/* Clear secrets first, since they don't get exported, and thus would
-	 * make the connection comparison below fail.
-	 */
-	remove_secrets (connection);
-	g_assert (nm_connection_compare (connection, reimported, NM_SETTING_COMPARE_FLAG_EXACT));
-
-	g_object_unref (reimported);
-	g_object_unref (connection);
-	g_free (path);
-}
-
 static void
 test_non_utf8_import (void)
 {
@@ -527,16 +588,18 @@ test_non_utf8_import (void)
 }
 
 static void
-test_static_key_import (void)
+test_static_key_import (gconstpointer test_data)
 {
 	_CREATE_PLUGIN (plugin);
 	NMConnection *connection;
 	NMSettingConnection *s_con;
 	NMSettingVpn *s_vpn;
-	const char *expected_id = "static";
+	const char *file, *expected_id, *expected_dir;
 	char *expected_path;
 
-	connection = get_basic_connection (plugin, SRCDIR, "static.ovpn");
+	nmtst_test_data_unpack (test_data, &file, &expected_id, &expected_dir);
+
+	connection = get_basic_connection (plugin, SRCDIR, file);
 	g_assert (connection);
 
 	/* Connection setting */
@@ -558,7 +621,7 @@ test_static_key_import (void)
 	_check_item (s_vpn, NM_OPENVPN_KEY_RENEG_SECONDS, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE, "10.11.12.13");
 	_check_item (s_vpn, NM_OPENVPN_KEY_PORT, NULL);
-	_check_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY_DIRECTION, "1");
+	_check_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY_DIRECTION, expected_dir);
 	_check_item (s_vpn, NM_OPENVPN_KEY_TA, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_TA_DIR, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_CIPHER, NULL);
@@ -577,41 +640,6 @@ test_static_key_import (void)
 	g_object_unref (connection);
 }
 
-#define STATIC_KEY_EXPORTED_NAME "static.ovpntest"
-static void
-test_static_key_export (void)
-{
-	_CREATE_PLUGIN (plugin);
-	NMConnection *connection;
-	NMConnection *reimported;
-	char *path;
-	gboolean success;
-	GError *error = NULL;
-
-	connection = get_basic_connection (plugin, SRCDIR, "static.ovpn");
-	g_assert (connection);
-
-	path = g_build_path ("/", TMPDIR, STATIC_KEY_EXPORTED_NAME, NULL);
-	success = nm_vpn_editor_plugin_export (plugin, path, connection, &error);
-	g_assert_no_error (error);
-	g_assert (success);
-
-	/* Now re-import it and compare the connections to ensure they are the same */
-	reimported = get_basic_connection (plugin, TMPDIR, STATIC_KEY_EXPORTED_NAME);
-	(void) unlink (path);
-	g_assert (reimported);
-
-	/* Clear secrets first, since they don't get exported, and thus would
-	 * make the connection comparison below fail.
-	 */
-	remove_secrets (connection);
-	g_assert (nm_connection_compare (connection, reimported, NM_SETTING_COMPARE_FLAG_EXACT));
-
-	g_object_unref (reimported);
-	g_object_unref (connection);
-	g_free (path);
-}
-
 static void
 test_port_import (gconstpointer test_data)
 {
@@ -642,6 +670,34 @@ test_port_import (gconstpointer test_data)
 	g_object_unref (connection);
 }
 
+static void
+test_connect_timeout_import (gconstpointer test_data)
+{
+	_CREATE_PLUGIN (plugin);
+	NMConnection *connection;
+	NMSettingConnection *s_con;
+	NMSettingVpn *s_vpn;
+	const char *file, *expected_timeout;
+
+	nmtst_test_data_unpack (test_data, &file, &expected_timeout);
+
+	connection = get_basic_connection (plugin, SRCDIR, file);
+	g_assert (connection);
+
+	/* Connection setting */
+	s_con = nm_connection_get_setting_connection (connection);
+	g_assert (s_con);
+
+	/* VPN setting */
+	s_vpn = nm_connection_get_setting_vpn (connection);
+	g_assert (s_vpn);
+
+	/* Data items */
+	_check_item (s_vpn, NM_OPENVPN_KEY_CONNECT_TIMEOUT, expected_timeout);
+
+	g_object_unref (connection);
+}
+
 static void
 test_ping_import (gconstpointer test_data)
 {
@@ -672,43 +728,6 @@ test_ping_import (gconstpointer test_data)
 	g_object_unref (connection);
 }
 
-static void
-test_port_export (gconstpointer test_data)
-{
-	_CREATE_PLUGIN (plugin);
-	NMConnection *connection;
-	NMConnection *reimported;
-	char *path;
-	gboolean success;
-	GError *error = NULL;
-	const char *file, *exported_name;
-
-	nmtst_test_data_unpack (test_data, &file, &exported_name);
-
-	connection = get_basic_connection (plugin, SRCDIR, file);
-	g_assert (connection);
-
-	path = g_build_path ("/", TMPDIR, exported_name, NULL);
-	success = nm_vpn_editor_plugin_export (plugin, path, connection, &error);
-	g_assert_no_error (error);
-	g_assert (success);
-
-	/* Now re-import it and compare the connections to ensure they are the same */
-	reimported = get_basic_connection (plugin, TMPDIR, exported_name);
-	(void) unlink (path);
-	g_assert (reimported);
-
-	/* Clear secrets first, since they don't get exported, and thus would
-	 * make the connection comparison below fail.
-	 */
-	remove_secrets (connection);
-	g_assert (nm_connection_compare (connection, reimported, NM_SETTING_COMPARE_FLAG_EXACT));
-
-	g_object_unref (reimported);
-	g_object_unref (connection);
-	g_free (path);
-}
-
 static void
 test_tun_opts_import (void)
 {
@@ -731,41 +750,6 @@ test_tun_opts_import (void)
 	g_object_unref (connection);
 }
 
-#define TUNOPTS_EXPORTED_NAME "tun-opts.ovpntest"
-static void
-test_tun_opts_export (void)
-{
-	_CREATE_PLUGIN (plugin);
-	NMConnection *connection;
-	NMConnection *reimported;
-	char *path;
-	gboolean success;
-	GError *error = NULL;
-
-	connection = get_basic_connection (plugin, SRCDIR, "tun-opts.conf");
-	g_assert (connection);
-
-	path = g_build_path ("/", TMPDIR, TUNOPTS_EXPORTED_NAME, NULL);
-	success = nm_vpn_editor_plugin_export (plugin, path, connection, &error);
-	g_assert_no_error (error);
-	g_assert (success);
-
-	/* Now re-import it and compare the connections to ensure they are the same */
-	reimported = get_basic_connection (plugin, TMPDIR, TUNOPTS_EXPORTED_NAME);
-	(void) unlink (path);
-	g_assert (reimported);
-
-	/* Clear secrets first, since they don't get exported, and thus would
-	 * make the connection comparison below fail.
-	 */
-	remove_secrets (connection);
-	g_assert (nm_connection_compare (connection, reimported, NM_SETTING_COMPARE_FLAG_EXACT));
-
-	g_object_unref (reimported);
-	g_object_unref (connection);
-	g_free (path);
-}
-
 static void
 test_proxy_http_import (void)
 {
@@ -787,7 +771,7 @@ test_proxy_http_import (void)
 	_check_item (s_vpn, NM_OPENVPN_KEY_COMP_LZO, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_FLOAT, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_RENEG_SECONDS, "0");
-	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE, "test.server.com:443");
+	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE, "[aa:bb::1]:1194:udp");
 	_check_item (s_vpn, NM_OPENVPN_KEY_PORT, "2352");
 	_check_item (s_vpn, NM_OPENVPN_KEY_CERT, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_KEY, NULL);
@@ -929,41 +913,6 @@ test_proxy_socks_import (void)
 	g_object_unref (connection);
 }
 
-#define PROXY_SOCKS_EXPORTED_NAME "proxy-socks.ovpntest"
-static void
-test_proxy_socks_export (void)
-{
-	_CREATE_PLUGIN (plugin);
-	NMConnection *connection;
-	NMConnection *reimported;
-	char *path;
-	gboolean success;
-	GError *error = NULL;
-
-	connection = get_basic_connection (plugin, SRCDIR, "proxy-socks.ovpn");
-	g_assert (connection);
-
-	path = g_build_path ("/", TMPDIR, PROXY_SOCKS_EXPORTED_NAME, NULL);
-	success = nm_vpn_editor_plugin_export (plugin, path, connection, &error);
-	g_assert_no_error (error);
-	g_assert (success);
-
-	/* Now re-import it and compare the connections to ensure they are the same */
-	reimported = get_basic_connection (plugin, TMPDIR, PROXY_SOCKS_EXPORTED_NAME);
-	(void) unlink (path);
-	g_assert (reimported);
-
-	/* Clear secrets first, since they don't get exported, and thus would
-	 * make the connection comparison below fail.
-	 */
-	remove_secrets (connection);
-	g_assert (nm_connection_compare (connection, reimported, NM_SETTING_COMPARE_FLAG_EXACT));
-
-	g_object_unref (reimported);
-	g_object_unref (connection);
-	g_free (path);
-}
-
 static void
 test_keysize_import (void)
 {
@@ -984,41 +933,6 @@ test_keysize_import (void)
 	g_object_unref (connection);
 }
 
-#define KEYSIZE_EXPORTED_NAME "keysize.ovpntest"
-static void
-test_keysize_export (void)
-{
-	_CREATE_PLUGIN (plugin);
-	NMConnection *connection;
-	NMConnection *reimported;
-	char *path;
-	gboolean success;
-	GError *error = NULL;
-
-	connection = get_basic_connection (plugin, SRCDIR, "keysize.ovpn");
-	g_assert (connection);
-
-	path = g_build_path ("/", TMPDIR, KEYSIZE_EXPORTED_NAME, NULL);
-	success = nm_vpn_editor_plugin_export (plugin, path, connection, &error);
-	g_assert_no_error (error);
-	g_assert (success);
-
-	/* Now re-import it and compare the connections to ensure they are the same */
-	reimported = get_basic_connection (plugin, TMPDIR, KEYSIZE_EXPORTED_NAME);
-	(void) unlink (path);
-	g_assert (reimported);
-
-	/* Clear secrets first, since they don't get exported, and thus would
-	 * make the connection comparison below fail.
-	 */
-	remove_secrets (connection);
-	g_assert (nm_connection_compare (connection, reimported, NM_SETTING_COMPARE_FLAG_EXACT));
-
-	g_object_unref (reimported);
-	g_object_unref (connection);
-	g_free (path);
-}
-
 static void
 test_device_import (gconstpointer test_data)
 {
@@ -1044,40 +958,26 @@ test_device_import (gconstpointer test_data)
 }
 
 static void
-test_device_export (gconstpointer test_data)
+test_mtu_disc_import (gconstpointer test_data)
 {
 	_CREATE_PLUGIN (plugin);
 	NMConnection *connection;
-	NMConnection *reimported;
-	char *path;
-	gboolean success;
-	GError *error = NULL;
-	const char *file, *exported_name;
+	NMSettingVpn *s_vpn;
+	const char *file, *expected_val;
 
-	nmtst_test_data_unpack (test_data, &file, &exported_name);
+	nmtst_test_data_unpack (test_data, &file, &expected_val);
 
 	connection = get_basic_connection (plugin, SRCDIR, file);
 	g_assert (connection);
 
-	path = g_build_path ("/", TMPDIR, exported_name, NULL);
-	success = nm_vpn_editor_plugin_export (plugin, path, connection, &error);
-	g_assert_no_error (error);
-	g_assert (success);
+	/* VPN setting */
+	s_vpn = nm_connection_get_setting_vpn (connection);
+	g_assert (s_vpn);
 
-	/* Now re-import it and compare the connections to ensure they are the same */
-	reimported = get_basic_connection (plugin, TMPDIR, exported_name);
-	(void) unlink (path);
-	g_assert (reimported);
+	/* Data items */
+	_check_item (s_vpn, NM_OPENVPN_KEY_MTU_DISC, expected_val);
 
-	/* Clear secrets first, since they don't get exported, and thus would
-	 * make the connection comparison below fail.
-	 */
-	remove_secrets (connection);
-	g_assert (nm_connection_compare (connection, reimported, NM_SETTING_COMPARE_FLAG_EXACT));
-
-	g_object_unref (reimported);
 	g_object_unref (connection);
-	g_free (path);
 }
 
 static void
@@ -1182,41 +1082,6 @@ test_route_import (void)
 	g_object_unref (connection);
 }
 
-#define ROUTE_EXPORTED_NAME "route.ovpntest"
-static void
-test_route_export (void)
-{
-	_CREATE_PLUGIN (plugin);
-	NMConnection *connection;
-	NMConnection *reimported;
-	char *path;
-	gboolean success;
-	GError *error = NULL;
-
-	connection = get_basic_connection (plugin, SRCDIR, "route.ovpn");
-	g_assert (connection);
-
-	path = g_build_path ("/", TMPDIR, ROUTE_EXPORTED_NAME, NULL);
-	success = nm_vpn_editor_plugin_export (plugin, path, connection, &error);
-	g_assert_no_error (error);
-	g_assert (success);
-
-	/* Now re-import it and compare the connections to ensure they are the same */
-	reimported = get_basic_connection (plugin, TMPDIR, ROUTE_EXPORTED_NAME);
-	(void) unlink (path);
-	g_assert (reimported);
-
-	/* Clear secrets first, since they don't get exported, and thus would
-	 * make the connection comparison below fail.
-	 */
-	remove_secrets (connection);
-	g_assert (nm_connection_compare (connection, reimported, NM_SETTING_COMPARE_FLAG_EXACT));
-
-	g_object_unref (reimported);
-	g_object_unref (connection);
-	g_free (path);
-}
-
 /*****************************************************************************/
 
 static void
@@ -1305,81 +1170,6 @@ test_args_parse_line (void)
 
 /*****************************************************************************/
 
-static void
-do_test_utils_str_utf8safe (const char *str, const char *expected)
-{
-	const char *str_safe, *s;
-	gs_free char *str2 = NULL;
-	gs_free char *str3 = NULL;
-
-	str_safe = nmv_utils_str_utf8safe_escape_c (str, &str2);
-
-	str3 = nmv_utils_str_utf8safe_escape (str);
-	g_assert_cmpstr (str3, ==, str_safe);
-	g_assert ((!str && !str3) || (str != str3));
-	g_clear_pointer (&str3, g_free);
-
-	if (expected == NULL) {
-		g_assert (str_safe == str);
-		g_assert (!str2);
-		if (str) {
-			g_assert (!strchr (str, '\\'));
-			g_assert (g_utf8_validate (str, -1, NULL));
-		}
-
-		g_assert (str == nmv_utils_str_utf8safe_unescape_c (str_safe, &str3));
-		g_assert (!str3);
-
-		str3 = nmv_utils_str_utf8safe_unescape (str_safe);
-		if (str) {
-			g_assert (str3 != str);
-			g_assert_cmpstr (str3, ==, str);
-		} else
-			g_assert (!str3);
-		g_clear_pointer (&str3, g_free);
-		return;
-	}
-
-	g_assert (str);
-	g_assert (str_safe != str);
-	g_assert (str_safe == str2);
-	g_assert (strchr (str, '\\') || !g_utf8_validate (str, -1, NULL));
-	g_assert (g_utf8_validate (str_safe, -1, NULL));
-
-	str3 = g_strcompress (str_safe);
-	g_assert_cmpstr (str, ==, str3);
-	g_clear_pointer (&str3, g_free);
-
-	str3 = nmv_utils_str_utf8safe_unescape (str_safe);
-	g_assert (str3 != str);
-	g_assert_cmpstr (str3, ==, str);
-	g_clear_pointer (&str3, g_free);
-
-	s = nmv_utils_str_utf8safe_unescape_c (str_safe, &str3);
-	g_assert (str3 != str);
-	g_assert (s == str3);
-	g_assert_cmpstr (str3, ==, str);
-	g_clear_pointer (&str3, g_free);
-
-	g_assert_cmpstr (str_safe, ==, expected);
-}
-
-static void
-test_utils_str_utf8safe (void)
-{
-	do_test_utils_str_utf8safe (NULL, NULL);
-	do_test_utils_str_utf8safe ("", NULL);
-	do_test_utils_str_utf8safe ("a", NULL);
-	do_test_utils_str_utf8safe ("ab", NULL);
-	do_test_utils_str_utf8safe ("abäb", NULL);
-	do_test_utils_str_utf8safe ("㈞abä㈞b", NULL);
-	do_test_utils_str_utf8safe ("Äab\\äb", "Äab\\\\äb");
-	do_test_utils_str_utf8safe ("ÄÄab\\äb", "ÄÄab\\\\äb");
-	do_test_utils_str_utf8safe ("Ä\304ab\\äb", "Ä\\304ab\\\\äb");
-}
-
-/*****************************************************************************/
-
 NMTST_DEFINE ();
 
 int main (int argc, char **argv)
@@ -1399,38 +1189,48 @@ int main (int argc, char **argv)
 #define _add_test_func_simple(func)       g_test_add_func ("/ovpn/properties/" #func, func)
 #define _add_test_func(detail, func, ...) nmtst_add_test_func ("/ovpn/properties/" detail, func, ##__VA_ARGS__)
 
+	_add_test_func_simple (test_nmovpn_remote_parse);
+
 	_add_test_func_simple (test_password_import);
-	_add_test_func_simple (test_password_export);
+	_add_test_func ("password-export", test_export_compare, "password.conf", "password.ovpntest");
 
 	_add_test_func_simple (test_tls_import);
 	_add_test_func_simple (test_tls_inline_import);
-	_add_test_func_simple (test_tls_export);
+	_add_test_func ("tls-export", test_export_compare, "tls.ovpn", "tls.ovpntest");
+
+	_add_test_func_simple (test_tls_import_2);
+	_add_test_func ("tls2-export", test_export_compare, "tls2.ovpn", "tls2.ovpntest");
 
 	_add_test_func_simple (test_pkcs12_import);
-	_add_test_func_simple (test_pkcs12_export);
+	_add_test_func ("pkcs12-export", test_export_compare, "pkcs12.ovpn", "pkcs12.ovpntest");
 
 	_add_test_func_simple (test_non_utf8_import);
 
-	_add_test_func_simple (test_static_key_import);
-	_add_test_func_simple (test_static_key_export);
+	_add_test_func ("static-import-1", test_static_key_import, "static.ovpn", "static", "1");
+	_add_test_func ("static-import-2", test_static_key_import, "static2.ovpn", "static2", "0");
+	_add_test_func ("static", test_export_compare, "static.ovpn", "static.ovpntest");
 
 	_add_test_func ("port-import", test_port_import, "port.ovpn", "port", "2345");
-	_add_test_func ("port-export", test_port_export, "port.ovpn", "port.ovpntest");
+	_add_test_func ("port-export", test_export_compare, "port.ovpn", "port.ovpntest");
 
 	_add_test_func ("rport-import", test_port_import, "rport.ovpn", "rport", "6789");
-	_add_test_func ("rport-export", test_port_export, "rport.ovpn", "rport.ovpntest");
+	_add_test_func ("rport-export", test_export_compare, "rport.ovpn", "rport.ovpntest");
+
+	_add_test_func ("connect-timeout-import", test_connect_timeout_import, "connect-timeout.ovpn", "19");
+	_add_test_func ("server-poll-timeout-import", test_connect_timeout_import, "server-poll-timeout.ovpn", "23");
+	_add_test_func ("connect-timeout-export", test_export_compare, "connect-timeout.ovpn", "connect-timeout.ovpntest");
 
 	_add_test_func_simple (test_tun_opts_import);
-	_add_test_func_simple (test_tun_opts_export);
+	_add_test_func ("tun-opts-export", test_export_compare, "tun-opts.conf", "tun-opts.ovpntest");
 
 	_add_test_func ("ping-with-exit-import", test_ping_import, "ping-with-exit.ovpn", "10", "120", NULL);
 	_add_test_func ("ping-with-restart-import", test_ping_import, "ping-with-restart.ovpn", "10", NULL, "30");
 
-	_add_test_func ("ping-with-exit-export", test_port_export, "ping-with-exit.ovpn", "ping-with-exit.ovpntest");
-	_add_test_func ("ping-with-restart-export", test_port_export, "ping-with-restart.ovpn", "ping-with-restart.ovpntest");
+	_add_test_func ("ping-with-exit-export", test_export_compare, "ping-with-exit.ovpn", "ping-with-exit.ovpntest");
+	_add_test_func ("ping-with-restart-export", test_export_compare, "ping-with-restart.ovpn", "ping-with-restart.ovpntest");
 
 	_add_test_func ("keepalive-import", test_ping_import, "keepalive.ovpn", "10", NULL, "30");
-	_add_test_func ("keepalive-export", test_port_export, "keepalive.ovpn", "keepalive.ovpntest");
+	_add_test_func ("keepalive-export", test_export_compare, "keepalive.ovpn", "keepalive.ovpntest");
 
 	_add_test_func_simple (test_proxy_http_import);
 	_add_test_func_simple (test_proxy_http_export);
@@ -1438,24 +1238,25 @@ int main (int argc, char **argv)
 	_add_test_func_simple (test_proxy_http_with_auth_import);
 
 	_add_test_func_simple (test_proxy_socks_import);
-	_add_test_func_simple (test_proxy_socks_export);
+	_add_test_func ("proxy-socks-export", test_export_compare, "proxy-socks.ovpn", "proxy-socks.ovpntest");
 
 	_add_test_func_simple (test_keysize_import);
-	_add_test_func_simple (test_keysize_export);
+	_add_test_func ("keysize-export", test_export_compare, "keysize.ovpn", "keysize.ovpntest");
 
 	_add_test_func ("device-import-default", test_device_import, "device.ovpn", "company0", "tun");
-	_add_test_func ("device-export-default", test_device_export, "device.ovpn", "device.ovpntest");
+	_add_test_func ("device-export-default", test_export_compare, "device.ovpn", "device.ovpntest");
 
 	_add_test_func ("device-import-notype", test_device_import, "device-notype.ovpn", "tap", NULL);
-	_add_test_func ("device-export-notype", test_device_export, "device-notype.ovpn", "device-notype.ovpntest");
+	_add_test_func ("device-export-notype", test_export_compare, "device-notype.ovpn", "device-notype.ovpntest");
+
+	_add_test_func ("mtu-disc-import", test_mtu_disc_import, "mtu-disc.ovpn", "yes");
+	_add_test_func ("mtu-disc-export", test_export_compare, "mtu-disc.ovpn", "mtu-disc.ovpntest");
 
 	_add_test_func_simple (test_route_import);
-	_add_test_func_simple (test_route_export);
+	_add_test_func ("route-export", test_export_compare, "route.ovpn", "route.ovpntest");
 
 	_add_test_func_simple (test_args_parse_line);
 
-	_add_test_func_simple (test_utils_str_utf8safe);
-
 	result = g_test_run ();
 	if (result != EXIT_SUCCESS)
 		return result;
